const { forwardTo } = require('prisma-binding')
const bcrypt = require('bcryptjs')
const jwt = require('jsonwebtoken')

const Query = {
  instruments: forwardTo('db'),
  projects: forwardTo('db'),
  projectVersions: forwardTo('db'),
  me: (parent, args, context, info) => {
    if (!context.request.userId) throw new Error('Not logged in.')
    return context.db.query.user({ where: { id: context.request.userId } }, info)
  }
}

const Mutation = {
  createUser: async (parent, args, context, info) => {
    const email = args.email.toLowerCase()
    const password = await bcrypt.hash(args.password, 10)
    console.log(email, password)
    const user = await context.db.mutation.createUser({
      data: {
        ...args,
        email,
        password
      }
    }, info)
    const token = jwt.sign({ userId: user.id }, process.env.APP_SECRET)
    context.response.cookie('token', token, {
      httpOnly: true,
      maxAge: 7 * 24 * 3600 * 1000
    })
    return user
  },
  userLogin: async (parent, args, context, info) => {
    const { email, password } = args
    const user = await context.db.query.user({ where: { email } })
    if (!user) throw new Error('User not found')
    const valid = await bcrypt.compare(password, user.password)
    if (!valid) throw new Error('Invalid password')
    const token = jwt.sign({ userId: user.id }, process.env.APP_SECRET)
    context.response.cookie('token', token, {
      httpOnly: true,
      maxAge: 7 * 24 * 3600 * 1000
    }, info)
    return user
  },
  userLogout: async (parent, args, context, info) => {
    context.response.clearCookie('token')
    return 'Logged out.'
  },
  createCharacterization: async (parent, args, context, info) => {
    const characterization = await context.db.mutation.createCharacterization({
      data: {
        name: args.name,
        projectVersion: {
          connect: { id: args.projectVersion }
        }
      }
    }, info)
    return characterization
  },
  createProject: async (parent, args, context, info) => {
    const project = await context.db.mutation.createProject({
      data: { ...args }
    }, info)
    return project
  },
  createProjectVersion: async (parent, args, context, info) => {
    console.log(args)
    const projectVersion = await context.db.mutation.createProjectVersion({
      data: {
        ...args,
        date: new Date(args.date).toISOString(),
        project: {
          connect: {
            id: args.project
          }
        },
        changes: {
          set: args.changes
        }
      }
    }, info)
    return projectVersion
  },
  uploadFiles: async (parent, args, context, info) => {
    console.log(args)
    const { name, description, files } = args
    let res
    for (const file of files) {
      res = await file
      console.log(res)
    }
    return res.filename
  }
}

const resolvers = {
  Query,
  Mutation
}

module.exports = { resolvers }
